/**
 * @file
 * Module JS logic.
 */

;(function ($, Drupal) {

  window.opignoQuizApp = {};

  Drupal.behaviors.opignoQuizAppFullScreen = {

    attach: function(context, settings) {
      if (opignoQuizApp._initialized == null) {
        for (var i = 0, len = Drupal.settings.opignoQuizApp.fullScreen.length; i < len; i++) {
          var node = Drupal.settings.opignoQuizApp.fullScreen[i],
              $form = $('form[action="' + Drupal.settings.basePath + 'node/' + node.nid + '/take"]', context); // @todo no clean url ??

          if ($form.length && !$form.hasClass('opigno-quiz-app-js-processed')) {
            $form.addClass('opigno-quiz-app-js-processed');
            opignoQuizApp.addFullscreenLinks($form.parent(), node);
          }
        }
        opignoQuizApp._initialized = true;
      }
    }
  }

  /**
   * Add the fullscreen links to the quiz navigation.
   *
   * @param {jQuery} $content
   */
  opignoQuizApp.addFullscreenLinks = function($content, node) {
    var $openLink = $('<a class="opigno-quiz-app-fullscreen-link opigno-quiz-app-go-fullscreen-link">' + Drupal.t("Go fullscreen") + '</a>'),
        $closeLink = $('<a class="opigno-quiz-app-fullscreen-link opigno-quiz-app-exit-fullscreen-link">' + Drupal.t("Exit fullscreen") + '</a>');
    
    var $quizNavigation = $content.find('#quiz_progress');
    // Prevent duplicates.
    $quizNavigation.find('a.opigno-quiz-app-fullscreen-link').remove();
    $quizNavigation.append($openLink).append($closeLink);

    $openLink.click(function() {
      opignoQuizApp.goFullScreen($content, node);
    });

    $closeLink.click(function() {
      opignoQuizApp.exitFullScreen($content._original || $content, true);
    });
  };

  /**
   * Take the content full screen.
   *
   * @param {jQuery} $content
   * @param {Object} node
   */
  opignoQuizApp.goFullScreen = function($content, node) {
    var $modal = opignoQuizApp.getModal(),
        $copy = $('<div></div>');

    $copy._original = $content;

    opignoQuizApp.stopVideos($content);

    $modal.append($copy);

    opignoQuizApp.fetchCurrentQuestion($copy, node);

    // Open the modal.
    $modal.animate({ opacity: 1 });
  };

  /**
   * Get the modal. If none exists, it will create one.
   *
   * @return {jQuery}
   */
  opignoQuizApp.getModal = function() {
    // Does a modal exist ?
    var $modal = $('#opigno-quiz-app-fullscreen-modal');
    if ($modal.length) {
      // @todo What do we do ? Empty it ?
      return $modal;
    }
    else {
      $('html').addClass('opigno-quiz-app-fullscreen');
      $modal = $('<div id="opigno-quiz-app-fullscreen-modal"></div>');
      $modal.appendTo('body');
      return $modal;
    }
  };

  /**
   * Hijack the "Next" and "Back" buttons, and load the content by Ajax.
   *
   * @param {jQuery} $content
   * @param {Object} node
   */
  opignoQuizApp.hijackNavigation = function($content, node) {
    $content.find('#edit-submit').click(function(e) {
        // Prevent other event listeners from crashing our party.
      e.stopImmediatePropagation();

      var $this = $(this);

      if (!$this.hasClass('loading')) {
        // Add loading class for styling.
        $this.addClass('loading');

        // Fetch the next question.
        opignoQuizApp.fetchNextQuestion($content, node);
      }

      // Prevent form submission.
      return false;
    });

    $content.find('#edit-back').click(function(e) {
      // Prevent other event listeners from crashing our party.
      e.stopImmediatePropagation();

      var $this = $(this);

      if (!$this.hasClass('loading')) {
        // Add loading class for styling.
        $this.addClass('loading');

        // Fetch the previous question.
        opignoQuizApp.fetchPrevQuestion($content, node);
      }

      // Prevent form submission.
      return false;
    });
  };

  /**
   * Load the current question via Ajax.
   * 
   * @param {jQuery} $content
   * @param {Object} node
   */
  opignoQuizApp.fetchCurrentQuestion = function($content, node) {
    $content.html('<div class="opigno-quiz-app-loading-content"></div>');
    
    $.ajax({
      url: Drupal.settings.basePath + 'node/' + node.nid + '/take/fullscreen/current',
      dataType: 'json',
      success: function(json) {
        if (json.success) {
          if (json.finished) {
            // Replace the actual content.
            $content._original.empty().html(json.content);

            // Close the modal, but don't reload the page.
            opignoQuizApp.exitFullScreen($content._original, false);
          }
          else {
            $content.empty().html(json.content);
            opignoQuizApp.addFullscreenLinks($content, node);
            opignoQuizApp.hijackNavigation($content, node);
            opignoQuizApp.handleVideos($content, node);
          }
        }
        else {
          alert(json.message);
        }
      }
    });
  };

  /**
   * Load the next question via Ajax.
   * 
   * @param {jQuery} $content
   * @param {Object} node
   */
  opignoQuizApp.fetchNextQuestion = function($content, node) {
    $content.html('<div class="opigno-quiz-app-loading-content"></div>');

    $.ajax({
      url: Drupal.settings.basePath + 'node/' + node.nid + '/take/fullscreen/next',
      dataType: 'json',
      success: function(json) {
        if (json.success) {
          if (json.finished) {
            // Replace the actual content.
            $content._original.empty().html(json.content);

            // Close the modal, but don't reload the page.
            opignoQuizApp.exitFullScreen($content._original, false);
          }
          else {
            $content.empty().html(json.content);
            opignoQuizApp.addFullscreenLinks($content, node);
            opignoQuizApp.hijackNavigation($content, node);
            opignoQuizApp.handleVideos($content, node);
          }
        }
        else {
          alert(json.message);
        }
      }
    });
  };

  /**
   * Load the previous question via Ajax.
   * 
   * @param {jQuery} $content
   * @param {Object} node
   */
  opignoQuizApp.fetchPrevQuestion = function($content, node) {
    $content.html('<div class="opigno-quiz-app-loading-content"></div>');

    $.ajax({
      url: Drupal.settings.basePath + 'node/' + node.nid + '/take/fullscreen/previous',
      dataType: 'json',
      success: function(json) {
        if (json.success) {
          $content.empty().html(json.content);
          opignoQuizApp.addFullscreenLinks($content, node);
          opignoQuizApp.hijackNavigation($content, node);
          opignoQuizApp.handleVideos($content, node);
        }
        else {
          alert(json.message);
        }
      }
    });
  };

  /**
   * Exit the fullscreen. Optionally reload the page, if a refresh is required.
   *
   * @param {jQuery} $content
   * @param {Boolean} reload
   */
  opignoQuizApp.exitFullScreen = function($content, reload) {
    reload = reload || false;

    if (reload && $content !== null) {
      // Change markup.
      $content.html('<div class="opigno-quiz-app-loading-content"></div>');
    }

    // Destroy modal.
    var $modal = opignoQuizApp.getModal();
    $modal.animate({ opacity: 0 }, { complete: function() {
      $modal.empty().remove();

      if (reload) {
        location.reload(true);
      }
    } });
  };

  /**
   * Handle dynamically loaded videos.
   *
   * @param {jQuery} $content
   * @param {Object} node
   */
  opignoQuizApp.handleVideos = function($content, node) {
    if (window.videojs) {
      $content.find('video').each(function() {
        var that = this;

        // Always generate a new ID.
        if (opignoQuizApp.handleVideos._count == null) {
          opignoQuizApp.handleVideos._count = 0;
        }
        that.id = 'node-' + node.nid + '-video-' + opignoQuizApp.handleVideos._count;
        opignoQuizApp.handleVideos._count++;

        videojs(that.id, { autoplay: "true" });
      });
    }
  }

  /**
   * Stop all videos.
   */
  opignoQuizApp.stopVideos = function($content) {
    $content.find('video').each(function() {
      if ($(this).parent().hasClass('video-js')) {
        videojs(this.id).pause();
      }
      else {
        this.pause();
      }
    });
  }

})(jQuery, Drupal);